<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * RealHomes Property Feed Admin Functions
 */
class RealHomes_Property_Feed_Admin {

	public function __construct() {

		add_action( 'admin_notices', array( $this, 'admin_error_notices') );

        add_action( 'admin_init', array( $this, 'admin_redirects' ) );

        add_filter( 'real_homes_sub_menus', array( $this, 'add_realhomes_property_feed_menu_item'), 10, 2 );

        add_filter( "plugin_action_links_" . plugin_basename( REALHOMES_PROPERTY_FEED_PLUGIN_FILE ), array( $this, 'plugin_add_settings_link' ) );

        add_action( 'admin_enqueue_scripts', array( $this, 'admin_styles' ), 5 );
        add_action( 'admin_enqueue_scripts', array( $this, 'admin_scripts' ), 5 );

        add_action( 'restrict_manage_posts', array( $this, 'restrict_manage_posts' ), 11 );
        add_filter( 'request', array( $this, 'request_query' ) );
	}

    /**
     * Output error messages when necessary
     */
    public function admin_error_notices() 
    {            
        global $wp_filesystem;
        
        if ( ! function_exists('WP_Filesystem') ) {
            require_once ABSPATH . 'wp-admin/includes/file.php';
        }
        WP_Filesystem(); // Initialize WP_Filesystem

        $error = '';    
        $uploads_dir = wp_upload_dir();

        if ( empty( $uploads_dir['error'] ) ) { // Ensure no upload directory errors
            $uploads_dir_import = trailingslashit( $uploads_dir['basedir'] ) . 'realhomes_property_feed_import/';
            $uploads_dir_export = trailingslashit( $uploads_dir['basedir'] ) . 'realhomes_property_feed_export/';

            // Create and check import folder
            if ( ! $wp_filesystem->exists( $uploads_dir_import ) ) {
                if ( ! $wp_filesystem->mkdir( $uploads_dir_import ) ) {
                    $error = esc_html__( 'Unable to create subdirectory in uploads folder for use by RealHomes Property Feed plugin. Please ensure the <a href="http://codex.wordpress.org/Changing_File_Permissions" target="_blank">correct permissions</a> are set.', 'realhomes-property-feed' );
                }
            } elseif ( ! $wp_filesystem->is_writable( $uploads_dir_import ) ) {
                $error = esc_html__( 'The uploads folder is not currently writable and will need to be before properties can be imported. Please ensure the <a href="http://codex.wordpress.org/Changing_File_Permissions" target="_blank">correct permissions</a> are set.', 'realhomes-property-feed' );
            }

            // Create and check export folder
            if ( ! $wp_filesystem->exists( $uploads_dir_export ) ) {
                if ( ! $wp_filesystem->mkdir( $uploads_dir_export ) ) {
                    $error = esc_html__( 'Unable to create subdirectory in uploads folder for use by RealHomes Property Feed plugin. Please ensure the <a href="http://codex.wordpress.org/Changing_File_Permissions" target="_blank">correct permissions</a> are set.', 'realhomes-property-feed' );
                }
            } elseif ( ! $wp_filesystem->is_writable( $uploads_dir_export ) ) {
                $error = esc_html__( 'The uploads folder is not currently writable and will need to be before properties can be exported. Please ensure the <a href="http://codex.wordpress.org/Changing_File_Permissions" target="_blank">correct permissions</a> are set.', 'realhomes-property-feed' );
            }
        } else {
            $error = esc_html__( 'An error occurred while trying to create the uploads folder. Please ensure the <a href="http://codex.wordpress.org/Changing_File_Permissions" target="_blank">correct permissions</a> are set.', 'realhomes-property-feed' ) . ' ' . esc_html( $uploads_dir['error'] );
        }

        // Ensure RealHomes theme is active
        if ( ! defined('INSPIRY_THEME_VERSION') ) { 
            $error = esc_html__( 'The RealHomes theme must be active to use the RealHomes Property Feed plugin.', 'realhomes-property-feed' );
        }
        
        if ( ! empty( $error ) ) {
            $allowed_html = array(
                'a' => array(
                    'href' => array(),
                    'target' => array(),
                ),
            );

            echo '<div class="error"><p><strong>' . wp_kses( $error, $allowed_html ) . '</strong></p></div>';
        }
    }


    /**
     * Handle redirects to import page after install.
     */
    public function admin_redirects()
    {
        // Setup wizard redirect
        if ( get_transient( '_realhomes_property_feed_activation_redirect' ) ) 
        {
            delete_transient( '_realhomes_property_feed_activation_redirect' );

            // Don't do redirect if part of multisite, doing batch-activate, or if no permission
            if ( is_network_admin() || isset( $_GET['activate-multi'] ) || ! current_user_can( 'manage_options' ) ) {
                return;
            }

            if ( function_exists('inspiry_theme_setup') )
            {   
                wp_safe_redirect( admin_url( 'admin.php?page=realhomes-property-feed-import' ) );
                exit;
            }
        }
    }

    public function add_realhomes_property_feed_menu_item( $submenus, $num )
    {
        $submenus['realhomes_import_properties'] = array(
            'real_homes',
            esc_html__( 'Import Properties', 'realhomes-property-feed' ),
            esc_html__( 'Import Properties', 'realhomes-property-feed' ),
            'manage_options',
            'realhomes-property-feed-import',
            array( $this, 'admin_page_import' )
        );

        $submenus['realhomes_export_properties'] = array(
            'real_homes',
            esc_html__( 'Export Properties', 'realhomes-property-feed' ),
            esc_html__( 'Export Properties', 'realhomes-property-feed' ),
            'manage_options',
            'realhomes-property-feed-export',
            array( $this, 'admin_page_export' )
        );

        return $submenus;
    }

    public function plugin_add_settings_link( $links )
    {
        $settings_link = '<a href="' . admin_url('admin.php?page=realhomes-property-feed-import') . '">' . __( 'Manage Imports', 'realhomes-property-feed' ) . '</a>';
        array_push( $links, $settings_link );

        $settings_link = '<a href="' . admin_url('admin.php?page=realhomes-property-feed-export') . '">' . __( 'Manage Exports', 'realhomes-property-feed' ) . '</a>';
        array_push( $links, $settings_link );

        $docs_link = '<a href="https://realhomespropertyfeed.com/documentation/" target="_blank">' . __( 'Docs', 'realhomes-property-feed' ) . '</a>';
        array_push( $links, $docs_link );

        if ( !class_exists( 'RealHomes_Property_Feed_Pro' ) )
        {
            $pro_link = '<a href="https://realhomespropertyfeed.com/pricing" target="_blank" style="font-weight:700; color:#93003c">' . __( 'Upgrade to PRO', 'realhomes-property-feed' ) . '</a>';
            array_push( $links, $pro_link );
        }

        return $links;
    }

    public function admin_page_import() 
    {
        global $wpdb, $post;

        $tabs = array(
            '' => __( 'Automatic Imports', 'realhomes-property-feed' ),
            'logs' => __( 'Logs', 'realhomes-property-feed' ),
            'settings' => __( 'Settings', 'realhomes-property-feed' ),
        );

        if ( !class_exists('RealHomes_Property_Feed_Pro') ) 
        {
            $tabs['pro'] = __( 'PRO Features', 'realhomes-property-feed' );
        }
        else
        {
            $tabs['license'] = __( 'License', 'realhomes-property-feed' );
        }

        $active_tab = ( isset($_GET['tab']) && !empty(sanitize_text_field(wp_unslash($_GET['tab']))) ) ? sanitize_text_field(wp_unslash($_GET['tab'])) : '';

        $options = get_option( 'realhomes_property_feed' , array() );

        include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-header.php' );

        switch ( $active_tab )
        {
            case "logs":
            {
                include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-primary-nav-import.php' );

                if ( 
                    isset($_GET['action']) && 
                    (
                        sanitize_text_field(wp_unslash($_GET['action'])) == 'view' ||
                        (
                            sanitize_text_field(wp_unslash($_GET['action'])) == 'search' && isset($_POST['log_search']) && sanitize_text_field(wp_unslash($_POST['log_search'])) != ''
                        )
                    )
                )
                {
                    switch ( sanitize_text_field(wp_unslash($_GET['action'])) )
                    {
                        case "view":
                        {
                            include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/class-realhomes-property-feed-admin-logs-view-import-table.php' );

                            $logs_view_table = new RealHomes_Property_Feed_Admin_Logs_View_Import_Table();
                            $logs_view_table->prepare_items();

                            $previous_instance = false;
                            $next_instance = false;

                            $sql = "
                                SELECT * 
                                FROM {$wpdb->prefix}realhomes_property_feed_logs_instance
                                INNER JOIN {$wpdb->prefix}realhomes_property_feed_logs_instance_log 
                                    ON {$wpdb->prefix}realhomes_property_feed_logs_instance.id = {$wpdb->prefix}realhomes_property_feed_logs_instance_log.instance_id
                                WHERE 
                            ";

                            $params = [];

                            if ( isset($_GET['import_id']) && !empty((int)$_GET['import_id']) ) 
                            {
                                $sql .= " import_id = %d AND ";
                                $params[] = (int)$_GET['import_id'];
                            }

                            $sql .= " instance_id < %d 
                                GROUP BY {$wpdb->prefix}realhomes_property_feed_logs_instance.id
                                ORDER BY start_date DESC
                                LIMIT 1";

                            $params[] = (int)$_GET['log_id'];

                            $logs = $wpdb->get_results($wpdb->prepare($sql, $params));
                            
                            if ( $logs )
                            {
                                foreach ( $logs as $log ) 
                                {
                                    $previous_instance = $log->instance_id;
                                }
                            }

                            $sql = "
                                SELECT * 
                                FROM {$wpdb->prefix}realhomes_property_feed_logs_instance
                                INNER JOIN {$wpdb->prefix}realhomes_property_feed_logs_instance_log 
                                    ON {$wpdb->prefix}realhomes_property_feed_logs_instance.id = {$wpdb->prefix}realhomes_property_feed_logs_instance_log.instance_id
                                WHERE 
                            ";

                            $params = [];

                            if ( isset($_GET['import_id']) && !empty((int)$_GET['import_id']) ) 
                            {
                                $sql .= " import_id = %d AND ";
                                $params[] = (int)$_GET['import_id'];
                            }

                            // Always add `instance_id > %d`
                            $sql .= " instance_id > %d 
                                GROUP BY {$wpdb->prefix}realhomes_property_feed_logs_instance.id
                                ORDER BY start_date ASC
                                LIMIT 1";

                            $params[] = (int)$_GET['log_id'];

                            // Securely prepare and execute the query
                            $logs = $wpdb->get_results($wpdb->prepare($sql, $params));

                            if ( $logs )
                            {
                                foreach ( $logs as $log ) 
                                {
                                    $next_instance = $log->instance_id;
                                }
                            }

                            include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-logs-view-import.php' );
                            break;
                        }
                        case "search":
                        {
                            include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/class-realhomes-property-feed-admin-logs-search-import-table.php' );

                            $log_tables = array();

                            $params = [];

                            $like_log_search = '%' . esc_like(sanitize_text_field(wp_unslash($_POST['log_search']))) . '%';

                            $query = "
                                SELECT " . $wpdb->prefix . "realhomes_property_feed_logs_instance_log.id
                                FROM " . $wpdb->prefix . "realhomes_property_feed_logs_instance
                                INNER JOIN 
                                    " . $wpdb->prefix . "realhomes_property_feed_logs_instance_log ON  " . $wpdb->prefix . "realhomes_property_feed_logs_instance.id = " . $wpdb->prefix . "realhomes_property_feed_logs_instance_log.instance_id
                                LEFT JOIN 
                                    " . $wpdb->posts . " ON " . $wpdb->posts . ".ID = " . $wpdb->prefix . "realhomes_property_feed_logs_instance_log.post_id
                                WHERE ";
                            if ( isset($_GET['import_id']) && !empty((int)$_GET['import_id']) )
                            {
                                $query .= " import_id = %d AND ";
                                $params[] = (int)$_GET['import_id'];
                            }
                            $query .= " (";
                            if ( isset($_POST['log_search']) && is_numeric($_POST['log_search']) )
                            {
                                $query .= " post_id = %s
                                        OR ";
                                $params[] = esc_sql(sanitize_text_field(wp_unslash($_POST['log_search'])));
                            }
                            $query .= "
                                        crm_id = %s
                                        OR 
                                        " . $wpdb->posts . ".post_title LIKE %s
                                    )
                                GROUP BY " . $wpdb->prefix . "realhomes_property_feed_logs_instance.id
                                ORDER BY start_date ASC
                            ";
                            $params[] = sanitize_text_field(wp_unslash($_POST['log_search']));
                            $params[] = $like_log_search;

                            $query = $wpdb->prepare($query, $params);

                            $log_results = $wpdb->get_results( 
                                $query
                            );

                            if ( $log_results )
                            {
                                foreach ( $log_results as $log_result ) 
                                {
                                    $logs_search_table = new RealHomes_Property_Feed_Admin_Logs_Search_Import_Table(array(), $log_result->id);
                                    $logs_search_table->prepare_items();

                                    $log_tables[] = $logs_search_table;
                                }
                            }

                            include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-logs-search-import.php' );
                            break;
                        }
                    }
                    
                }
                else
                {
                    include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/class-realhomes-property-feed-admin-logs-import-table.php' );

                    $logs_table = new RealHomes_Property_Feed_Admin_Logs_Import_Table();
                    $logs_table->prepare_items();

                    include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-logs-import.php' );
                }

                break;
            }
            case "settings":
            {
                include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-primary-nav-import.php' );
                include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-settings-import.php' );
                break;
            }
            case "license":
            {
                $license_key_status = apply_filters( 'realhomes_property_feed_pro_status', array() );

                include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-primary-nav-import.php' );
                include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-license.php' );
                break;
            }
            case "pro":
            {
                include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-primary-nav-import.php' );

                $features = $this->get_pro_features();

                include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-pro.php' );
                break;
            }
            default:
            {
                $active_tab = ( isset($_GET['action']) && !empty(sanitize_text_field(wp_unslash($_GET['action']))) ) ? sanitize_text_field(wp_unslash($_GET['action'])) : '';

                if ( $active_tab == 'addimport' || $active_tab == 'editimport' )
                {
                    $import_id = ( isset($_GET['import_id']) && !empty(sanitize_text_field(wp_unslash($_GET['import_id']))) ) ? (int)$_GET['import_id'] : false;

                    $frequencies = realhomes_property_feed_get_import_frequencies();

                    $import_settings = array();
                    if ( $active_tab == 'editimport' )
                    {
                        $imports = ( isset($options['imports']) && is_array($options['imports']) && !empty($options['imports']) ) ? $options['imports'] : array();
                        if ( isset($imports[$import_id]) )
                        {
                            $import_settings = $imports[$import_id];

                            // ensure frequency is not a PRO one if PRO not enabled
                            if ( apply_filters( 'realhomes_property_feed_pro_active', false ) !== true )
                            {
                                if ( isset($frequencies[$import_settings['frequency']]['pro']) && $frequencies[$import_settings['frequency']]['pro'] === true )
                                {
                                    $import_settings['frequency'] = 'daily';
                                }
                            }
                        }
                    }

                    // get WP users / authors
                    $wp_users = array();

                    $users = get_users( array( 'orderby' => 'name' ) );
                    foreach ( $users as $user ) 
                    {
                        $wp_users[$user->ID] = $user->display_name;
                    }

                    // get agents
                    $realhomes_agents = array();

                    $args = array(
                        'post_type' => 'agent',
                        'nopaging' => true
                    );

                    $agent_query = new WP_Query( $args );

                    if ( $agent_query->have_posts() )
                    {
                        while ( $agent_query->have_posts() )
                        {
                            $agent_query->the_post();

                            $realhomes_agents[get_the_ID()] = get_the_title();
                        }
                    }
                    wp_reset_postdata();

                    $formats = realhomes_property_feed_get_import_formats();

                    include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-import-settings.php' );
                }
                else
                {
                    include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-primary-nav-import.php' );

                    include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/class-realhomes-property-feed-admin-automatic-imports-table.php' );

                    $automatic_imports_table = new RealHomes_Property_Feed_Admin_Automatic_Imports_Table();
                    $automatic_imports_table->prepare_items();

                    $run_now_button = false;
                    $imports = ( isset($options['imports']) && is_array($options['imports']) && !empty($options['imports']) ) ? $options['imports'] : array();
                    foreach ( $imports as $import_id => $import_settings )
                    {
                        if ( !isset($import_settings['running']) || ( isset($import_settings['running']) && $import_settings['running'] !== true ) )
                        {
                            continue;
                        }

                        if ( isset($import_settings['deleted']) && $import_settings['deleted'] === true )
                        {
                            continue;
                        }

                        $run_now_button = true;
                        continue;
                    }

                    include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-automatic-imports.php' );
                }
            }
        }

        include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-footer.php' );
    }

    public function admin_page_export() 
    {
        global $wpdb, $post;

        $tabs = array(
            '' => __( 'Automatic Exports', 'realhomes-property-feed' ),
            'logs' => __( 'Logs', 'realhomes-property-feed' ),
            'settings' => __( 'Settings', 'realhomes-property-feed' ),
        );

        if ( !class_exists('RealHomes_Property_Feed_Pro') ) 
        {
            $tabs['pro'] = __( 'PRO Features', 'realhomes-property-feed' );
        }
        else
        {
            $tabs['license'] = __( 'License', 'realhomes-property-feed' );
        }

        $active_tab = ( isset($_GET['tab']) && !empty(sanitize_text_field(wp_unslash($_GET['tab']))) ) ? sanitize_text_field(wp_unslash($_GET['tab'])) : '';

        $options = get_option( 'realhomes_property_feed' , array() );

        include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-header.php' );

        switch ( $active_tab )
        {
            case "logs":
            {
                include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-primary-nav-export.php' );

                if ( isset($_GET['action']) && sanitize_text_field(wp_unslash($_GET['action'])) == 'view' )
                {
                    include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/class-realhomes-property-feed-admin-logs-view-export-table.php' );

                    $logs_view_table = new RealHomes_Property_Feed_Admin_Logs_View_Export_Table();
                    $logs_view_table->prepare_items();

                    $previous_instance = false;
                    $next_instance = false;

                    $params = [];

                    $query = "
                        SELECT * 
                        FROM " . $wpdb->prefix . "realhomes_property_feed_export_logs_instance
                        INNER JOIN 
                            " . $wpdb->prefix . "realhomes_property_feed_export_logs_instance_log ON  " . $wpdb->prefix . "realhomes_property_feed_export_logs_instance.id = " . $wpdb->prefix . "realhomes_property_feed_export_logs_instance_log.instance_id
                        WHERE ";
                    if ( isset($_GET['export_id']) && !empty((int)$_GET['export_id']) )
                    {
                        $query .= " export_id = %d AND ";
                        $params[] = (int)$_GET['export_id'];
                    }
                    $query .= "
                        instance_id < %d
                        GROUP BY " . $wpdb->prefix . "realhomes_property_feed_export_logs_instance.id
                        ORDER BY start_date DESC
                        LIMIT 1
                    ";
                    $params[] = (int)$_GET['log_id'];

                    $logs = $wpdb->get_results( 
                        $wpdb->prepare($query, $params)
                    );

                    if ( $logs )
                    {
                        foreach ( $logs as $log ) 
                        {
                            $previous_instance = $log->instance_id;
                        }
                    }

                    $logs = $wpdb->get_results( 
                        $wpdb->prepare("
                        SELECT * 
                        FROM " . $wpdb->prefix . "realhomes_property_feed_export_logs_instance
                        INNER JOIN 
                            " . $wpdb->prefix . "realhomes_property_feed_export_logs_instance_log ON  " . $wpdb->prefix . "realhomes_property_feed_export_logs_instance.id = " . $wpdb->prefix . "realhomes_property_feed_export_logs_instance_log.instance_id
                        WHERE 
                             " . ( ( isset($_GET['export_id']) && !empty((int)$_GET['export_id']) ) ? " export_id = '" . (int)$_GET['export_id'] . "' AND " : "" ) . "
                            instance_id > %d
                        GROUP BY " . $wpdb->prefix . "realhomes_property_feed_export_logs_instance.id
                        ORDER BY start_date ASC
                        LIMIT 1
                        ", (int)$_GET['log_id'])
                    );

                    if ( $logs )
                    {
                        foreach ( $logs as $log ) 
                        {
                            $next_instance = $log->instance_id;
                        }
                    }

                    include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-logs-view-export.php' );
                }
                else
                {
                    include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/class-realhomes-property-feed-admin-logs-export-table.php' );

                    $logs_table = new RealHomes_Property_Feed_Admin_Logs_Export_Table();
                    $logs_table->prepare_items();

                    include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-logs-export.php' );
                }

                break;
            }
            case "license":
            {
                $license_key_status = apply_filters( 'realhomes_property_feed_pro_status', array() );

                include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-primary-nav-export.php' );
                include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-license.php' );
                break;
            }
            case "settings":
            {
                include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-primary-nav-export.php' );
                include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-settings-export.php' );
                break;
            }
            case "pro":
            {
                include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-primary-nav-export.php' );

                $features = $this->get_pro_features();

                include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-pro.php' );
                break;
            }
            default:
            {
                $active_tab = ( isset($_GET['action']) && !empty(sanitize_text_field(wp_unslash($_GET['action']))) ) ? sanitize_text_field(wp_unslash($_GET['action'])) : '';

                if ( $active_tab == 'addexport' || $active_tab == 'editexport' )
                {
                    $export_id = ( isset($_GET['export_id']) && !empty(sanitize_text_field(wp_unslash($_GET['export_id']))) ) ? (int)$_GET['export_id'] : false;

                    $export_settings = array();
                    if ( $active_tab == 'editexport' )
                    {
                        $exports = ( isset($options['exports']) && is_array($options['exports']) && !empty($options['exports']) ) ? $options['exports'] : array();
                        if ( isset($exports[$export_id]) )
                        {
                            $export_settings = $exports[$export_id];
                        }
                    }

                    $formats = realhomes_property_feed_get_export_formats();

                    $frequencies = realhomes_property_feed_get_export_frequencies();

                    include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-export-settings.php' );
                }
                else
                {
                    include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-primary-nav-export.php' );

                    include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/class-realhomes-property-feed-admin-automatic-exports-table.php' );

                    $automatic_exports_table = new RealHomes_Property_Feed_Admin_Automatic_Exports_Table();
                    $automatic_exports_table->prepare_items();

                    $run_now_button = false;
                    $exports = ( isset($options['exports']) && is_array($options['exports']) && !empty($options['exports']) ) ? $options['exports'] : array();
                    
                    // remove any non-cron formats
                    foreach ( $exports as $export_id => $export_settings  )
                    {
                        $format = realhomes_property_feed_get_format_from_export_id( $export_id );
                        if ( isset($format['method']) && ( $format['method'] == 'cron' || $format['method'] == 'url' ) )
                        {

                        }
                        else
                        {
                            unset($exports[$export_id]);
                        }
                    }

                    foreach ( $exports as $export_id => $export_settings )
                    {
                        if ( !isset($export_settings['running']) || ( isset($export_settings['running']) && $export_settings['running'] !== true ) )
                        {
                            continue;
                        }

                        if ( isset($export_settings['deleted']) && $export_settings['deleted'] === true )
                        {
                            continue;
                        }

                        $run_now_button = true;
                        continue;
                    }

                    include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-automatic-exports.php' );
                }
            }
        }

        include( dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/views/admin-settings-footer.php' );
    }

    private function get_pro_features()
    {
        return array(
            array(
                'icon' => 'dashicons dashicons-admin-multisite',
                'title' => 'Import unlimited properties',
                'description' => 'Remove the 25 property limit and import/export unlimited properties.'
            ),
            array(
                'icon' => 'dashicons dashicons-clock',
                'title' => 'Imports ran more frequently',
                'description' => 'Choose from daily, twice daily, hourly or every 15 minutes meaning your properties import quicker.'
            ),
            array(
                'icon' => 'dashicons dashicons-admin-comments',
                'title' => 'Priority support',
                'description' => 'Our UK based friendly support team are on hand to answer any of your questions and assist with setting up feeds.'
            ),
            array(
                'icon' => 'dashicons dashicons-database-import',
                'title' => 'Multiple imports and exports',
                'description' => 'Have multiple simultaneous imports and exports running at once. Useful if importing from or exporting to multiple sources.'
            ),
            array(
                'icon' => 'dashicons dashicons-email',
                'title' => 'Email reports',
                'description' => 'Get a report emailed to you each time an import has finished running.'
            ),
            array(
                'icon' => 'dashicons dashicons-database-add',
                'title' => 'Store logs for longer',
                'description' => 'We\'ll store logs for up to seven days making debugging any issues much easier.'
            ),
            array(
                'icon' => 'dashicons dashicons-admin-media',
                'title' => 'Save disk space over time',
                'description' => 'Choose to delete property media when a property comes off of the market to save on disk space.'
            ),
            array(
                'icon' => 'dashicons dashicons-database',
                'title' => 'Download and process media separately',
                'description' => 'Allow imports to complete quicker by opting to process media in a separate background queue.'
            ),
            array(
                'icon' => 'dashicons dashicons-yes',
                'title' => 'Select which properties are exported',
                'description' => 'Get control over which properties are sent to which portals.'
            ),
        );
    }

    /**
     * Enqueue styles
     */
    public function admin_styles() 
    {
        global $wp_scripts;

        if ( isset($_GET['page']) && ( sanitize_text_field(wp_unslash($_GET['page'])) == 'realhomes-property-feed-import' || sanitize_text_field(wp_unslash($_GET['page'])) == 'realhomes-property-feed-export' ) ) 
        {
            wp_enqueue_style( 'realhomes_property_feed_admin_styles', untrailingslashit( plugins_url( '/', REALHOMES_PROPERTY_FEED_PLUGIN_FILE ) ) . '/assets/css/admin.css', array(), REALHOMES_PROPERTY_FEED_VERSION );
            wp_enqueue_style( 'select2_styles', untrailingslashit( plugins_url( '/', REALHOMES_PROPERTY_FEED_PLUGIN_FILE ) ) . '/assets/css/select2.min.css', array(), '4.0.13' );
        }
    }


    /**
     * Enqueue scripts
     */
    public function admin_scripts() 
    {
        global $wp_query, $post, $tabs;

        $statuses = array();
        $property_types = array();

        if ( isset($_GET['page']) && ( sanitize_text_field(wp_unslash($_GET['page'])) == 'realhomes-property-feed-import' || sanitize_text_field(wp_unslash($_GET['page'])) == 'realhomes-property-feed-export' ) ) 
        {
            // scripts used throughout import and export
            wp_register_script( 'select2', untrailingslashit( plugins_url( '/', REALHOMES_PROPERTY_FEED_PLUGIN_FILE ) ) . '/assets/js/select2.min.js', array( 'jquery' ), '4.0.13' );
            wp_enqueue_script( 'select2' );
        }

        if ( isset($_GET['page']) && sanitize_text_field(wp_unslash($_GET['page'])) == 'realhomes-property-feed-import' ) 
        {
            $terms = get_terms( array(
                'taxonomy'   => 'property-status',
                'hide_empty' => false,
            ) );

            if ( is_array($terms) && !empty($terms) )
            {
                foreach ( $terms as $term )
                {
                    $statuses[$term->term_id] = $term->name;
                }
            }

            $terms = get_terms( array(
                'taxonomy'   => 'property-type',
                'hide_empty' => false,
            ) );

            if ( is_array($terms) && !empty($terms) )
            {
                foreach ( $terms as $term )
                {
                    $property_types[$term->term_id] = $term->name;
                }
            }

            // enqueue draggable/droppable for XML field mapping
            wp_enqueue_script( 'jquery-ui-draggable' );
            wp_enqueue_script( 'jquery-ui-droppable' );

            wp_register_script( 'realhomes_property_feed_admin_import_script', untrailingslashit( plugins_url( '/', REALHOMES_PROPERTY_FEED_PLUGIN_FILE ) ) . '/assets/js/admin-import.js', array( 'jquery' ), REALHOMES_PROPERTY_FEED_VERSION );

            $formats = realhomes_property_feed_get_import_formats();

            $import_id = ( isset($_GET['import_id']) && !empty(sanitize_text_field(wp_unslash($_GET['import_id']))) ) ? (int)$_GET['import_id'] : false;

            $import_settings = array();

            if ( $import_id !== false )
            {
                $options = get_option( 'realhomes_property_feed' , array() );

                $imports = ( isset($options['imports']) && is_array($options['imports']) && !empty($options['imports']) ) ? $options['imports'] : array();
                if ( isset($imports[$import_id]) )
                {
                    $import_settings = $imports[$import_id];
                }
            }

            // get fields referenced in import format file to show warning if field also gets mapped in 'Field Mapping' section
            $format_fields_imported_by_default = array();
            foreach ( $formats as $key => $format )
            {
                if ( !isset($formats[$key]['realhomes_fields_imported_by_default']) )
                {
                    $formats[$key]['realhomes_fields_imported_by_default'] = array();
                }

                $format_file = dirname(REALHOMES_PROPERTY_FEED_PLUGIN_FILE) . '/includes/import-formats/class-realhomes-property-feed-format-' . $key . '.php';
                $format_file = str_replace( array( "_local", "_remote" ), "", $format_file);
                $format_file = str_replace( "_", "-", $format_file);
                if ( file_exists($format_file) )
                {
                    $import_file_contents = file_get_contents($format_file);

                    if ( !empty($import_file_contents) )
                    {
                        // get all 'fave_X' fields already imported in format
                        preg_match_all('/\'fave_(.*?)[\']+/', $import_file_contents, $matches);
                        if ( isset($matches[1]) && is_array($matches[1]) && !empty($matches[1]) )
                        {
                            foreach ( $matches[1] as $match )
                            {
                                $formats[$key]['realhomes_fields_imported_by_default'][] = 'fave_' . $match;
                            }
                        }

                        // get all taxonomies already mapped
                        preg_match_all('/wp_set_object_terms\((.*?)[\)]+[;]+/', $import_file_contents, $matches);
                        if ( isset($matches[1]) && is_array($matches[1]) && !empty($matches[1]) )
                        {
                            foreach ( $matches[1] as $match )
                            {
                                $explode_set_object_terms = explode(",", $match);
                                if ( count($explode_set_object_terms) >= 3 )
                                {
                                    $taxonomy = str_replace(array('"', "'"), "", trim($explode_set_object_terms[2]));
                                    $formats[$key]['realhomes_fields_imported_by_default'][] = trim($taxonomy);
                                }
                            }
                        }
                    }
                    else
                    {
                        //echo 'File ' . $format_file . ' contents empty';
                    }
                }
                else
                {
                    //echo 'File ' . $format_file . ' not found';
                }

                if ( $key != 'xml' && $key != 'csv' )
                {
                    $formats[$key]['realhomes_fields_imported_by_default'][] = 'post_title';
                    $formats[$key]['realhomes_fields_imported_by_default'][] = 'post_excerpt';
                    $formats[$key]['realhomes_fields_imported_by_default'][] = 'post_content';
                }

                $formats[$key]['realhomes_fields_imported_by_default'] = array_unique($formats[$key]['realhomes_fields_imported_by_default']);
                $formats[$key]['realhomes_fields_imported_by_default'] = array_filter($formats[$key]['realhomes_fields_imported_by_default']);
            }

            wp_localize_script( 'realhomes_property_feed_admin_import_script', 'rhpf_admin_object', array( 
                'action' => ( ( isset($_GET['action']) && in_array(sanitize_text_field(wp_unslash($_GET['action'])), array('addimport', 'editimport')) ) ? sanitize_text_field(wp_unslash($_GET['action'])) : '' ),
                'formats' => $formats,
                'import_settings' => $import_settings,
                'statuses' => $statuses,
                'property_types' => $property_types,
                'realhomes_fields_for_field_mapping' => realhomes_property_feed_get_fields_for_field_mapping(),
                'ajax_nonce' => wp_create_nonce("rhpf_ajax_nonce"),
                'table_order' => ( isset($_GET['order']) ? sanitize_text_field(wp_unslash($_GET['order'])) : '' ),
                'table_orderby' => ( isset($_GET['orderby']) ? sanitize_text_field(wp_unslash($_GET['orderby'])) : '' ),
                'table_rhpf_filter' => ( isset($_GET['rhpf_filter']) ? sanitize_text_field(wp_unslash($_GET['rhpf_filter'])) : '' ),
                'table_rhpf_filter_format' => ( isset($_GET['rhpf_filter_format']) ? sanitize_text_field(wp_unslash($_GET['rhpf_filter_format'])) : '' ),
                'table_refresh_automatic_imports' => apply_filters( 'realhomes_property_feed_table_refresh_automatic_imports', 30000 ),
                'table_refresh_status_interval' => apply_filters( 'realhomes_property_feed_table_refresh_status_interval', 5000 )
            ) );

            wp_enqueue_script( 'realhomes_property_feed_admin_import_script' );
        }

        if ( isset($_GET['page']) && sanitize_text_field(wp_unslash($_GET['page'])) == 'realhomes-property-feed-export' ) 
        {
            wp_register_script( 'realhomes_property_feed_admin_export_script', untrailingslashit( plugins_url( '/', REALHOMES_PROPERTY_FEED_PLUGIN_FILE ) ) . '/assets/js/admin-export.js', array( 'jquery' ), REALHOMES_PROPERTY_FEED_VERSION );

            $formats = realhomes_property_feed_get_export_formats();

            $export_id = ( isset($_GET['export_id']) && !empty(sanitize_text_field(wp_unslash($_GET['export_id']))) ) ? (int)$_GET['export_id'] : false;

            $export_settings = array();

            if ( $export_id !== false )
            {
                $options = get_option( 'realhomes_property_feed' , array() );

                $exports = ( isset($options['exports']) && is_array($options['exports']) && !empty($options['exports']) ) ? $options['exports'] : array();
                if ( isset($exports[$export_id]) )
                {
                    $export_settings = $exports[$export_id];
                }
            }

            wp_localize_script( 'realhomes_property_feed_admin_export_script', 'rhpf_admin_object', array( 
                'formats' => $formats,
                'export_settings' => $export_settings,
            ) );

            wp_enqueue_script( 'realhomes_property_feed_admin_export_script' );
        }
    }

    /**
     * Filters for post types
     */
    public function restrict_manage_posts() 
    {
        global $typenow, $wp_query;

        if ( $typenow == 'property' )
        {
            $options = get_option( 'realhomes_property_feed' , array() );
            $imports = ( isset($options['imports']) && is_array($options['imports']) && !empty($options['imports']) ) ? $options['imports'] : array();

            if ( !empty($imports) )
            {
                uasort($imports, function($a, $b)
                {
                    return $a['format'] <=> $b['format'];
                });

                $output = '<select name="_import_id" id="dropdown_rhpf_import_id">';

                $output .= '<option value="">' . esc_html( __( 'All Imports', 'realhomes-property-feed' ) ) . '</option>';

                foreach ( $imports as $key => $import )
                {
                    if ( 
                        isset($imports[$key]['deleted']) && $imports[$key]['deleted'] === true &&
                        apply_filters( 'realhomes_property_feed_include_deleted_in_filter', true ) === false
                    )
                    {
                        continue;
                    }

                    $format = realhomes_property_feed_get_import_format( $import['format'] );

                    $output .= '<option value="' . esc_attr($key) . '"';
                    if ( isset($_GET['_import_id']) && !empty($_GET['_import_id']) && is_numeric($_GET['_import_id']) && (int)$_GET['_import_id'] == $key ) 
                    {
                        $output .= ' selected';
                    }
                    $output .= '>' . esc_html( isset($format['name']) ? $format['name'] : '-' );
                    $output .= ' (' . esc_html($key) . ')';
                    if ( isset($imports[$key]['deleted']) && $imports[$key]['deleted'] === true )
                    {
                        $output .= ' - ' . esc_html( 'Deleted', 'realhomes-property-feed' );
                    }
                    else
                    {
                        if ( isset($import['running']) && $import['running'] === true )
                        {
                            $output .= ' - ' . esc_html( 'Active', 'realhomes-property-feed' );
                        }
                        else
                        {
                            $output .= ' - ' . esc_html( 'Inactive', 'realhomes-property-feed' );
                        }
                    }

                    $output .= '</option>';
                }
                $output .= '</select>';

                echo $output;

            }
        }
    }

    /**
     * Filters handler
     * @param  array $vars
     * @return array
     */
    public function request_query( $vars )
    {
        global $typenow, $wp_query;

        if ( !isset($vars['meta_query']) ) { $vars['meta_query'] = array(); }

        if ( 'property' === $typenow ) 
        {
            if ( isset($_GET['_import_id']) && !empty($_GET['_import_id']) && is_numeric($_GET['_import_id']) ) 
            {
                $import_id = (int)$_GET['_import_id'];

                $vars['meta_query'][] = array(
                    'key' => '_imported_ref_' . $import_id,
                    'compare' => 'EXISTS'
                );
            }
        }

        return $vars;
    }
}

new RealHomes_Property_Feed_Admin();