<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * RealHomes Property Feed Export Functions
 */
class RealHomes_Property_Feed_Export {

	public function __construct() {

        add_action( 'admin_init', array( $this, 'check_not_multiple_if_no_pro') );

        add_action( 'admin_init', array( $this, 'save_export_settings') );

        add_action( 'admin_init', array( $this, 'toggle_export_running_status') );

        add_action( 'admin_init', array( $this, 'delete_export') );

        add_filter( 'realhomes_property_feed_export_property_data', array( $this, 'perform_field_mapping' ), 1, 3 );
	}

    public function check_not_multiple_if_no_pro()
    {
        if ( isset($_GET['action']) && $_GET['action'] == 'addexport' )
        {
            if ( apply_filters( 'realhomes_property_feed_pro_active', false ) !== true ) 
            {
                $options = get_option( 'realhomes_property_feed', array() );
                $exports = ( isset($options['exports']) && is_array($options['exports']) && !empty($options['exports']) ) ? $options['exports'] : array();

                foreach ( $exports as $key => $export )
                {
                    if ( $exports[$key]['deleted'] && $exports[$key]['deleted'] === true )
                    {
                        unset( $exports[$key] );
                    }
                }

                if ( count($exports) >=1 )
                {
                    wp_redirect( admin_url( 'admin.php?page=realhomes-property-feed-export&rhpferrormessage=' . urlencode(__( 'Maximum number of exports reached. Upgrade to PRO if wanting to benfit from multiple exports and more', 'realhomes-property-feed' ) ) ) );
                    die();
                }
            }
        }
    }

    public function save_export_settings()
    {
        if ( !isset($_POST['save_export_settings']) )
        {
            return;
        }

        if ( !isset($_POST['_wpnonce']) || ( isset($_POST['_wpnonce']) && !wp_verify_nonce( sanitize_text_field(wp_unslash($_POST['_wpnonce'])), 'save-export-settings' ) ) ) 
        {
            die( esc_html(__( "Failed security check", 'realhomes-property-feed' )) );
        }

        // ready to save
        $export_id = !empty($_POST['export_id']) ? (int)$_POST['export_id'] : time();

        $options = get_option( 'realhomes_property_feed' , array() );
        if ( !is_array($options) ) { $options = array(); }
        if ( !is_array($options['exports']) ) { $options['exports'] = array(); }
        if ( !is_array($options['exports'][$export_id]) ) { $options['exports'][$export_id] = array(); }

        $format = isset($_POST['format']) ? sanitize_text_field(wp_unslash($_POST['format'])) : '';

        $running = ( isset($_POST['running']) && sanitize_text_field(wp_unslash($_POST['running'])) == 'yes' ) ? true : false;

        $export_options = array(
            'running' => $running,
            'format' => $format,
            'name' => sanitize_text_field(wp_unslash($_POST['export_name'])),
            'frequency' => sanitize_text_field(wp_unslash($_POST['frequency'])), // might want to validate this is not a pro frequency
        );

        $rules = array();
        if ( 
            isset($_POST['field_mapping_rules']) && 
            is_array($_POST['field_mapping_rules']) && 
            count($_POST['field_mapping_rules']) > 1 // more than 1 to ignore template
        )
        {
            $rule_i = 0;
            foreach ( $_POST['field_mapping_rules'] as $j => $field )
            {
                if ($j !== '{rule_count}') // ignore template
                {
                    $rules[$rule_i] = array(
                        'field' => sanitize_text_field($field['field']),
                        'result' => sanitize_text_field($field['result']),
                        'rules' => array(),
                    );

                    unset($field['field']);
                    unset($field['result']);

                    foreach ( $field as $i => $rule_fields )
                    {   
                        foreach ( $rule_fields as $k => $rule_field )
                        {
                            $rules[$rule_i]['rules'][$k][$i] = sanitize_text_field($rule_field);
                        }
                    }

                    ++$rule_i;
                }
            }
        }
        $export_options['field_mapping_rules'] = $rules;

        // Save core format fields (FTP Details etc)
        $formats = realhomes_property_feed_get_export_formats();
        if ( isset($formats[$format]) )
        {
            if ( isset($formats[$format]['fields']) && !empty($formats[$format]['fields']) )
            {
                foreach ( $formats[$format]['fields'] as $field )
                {   
                    if ( isset($field['id']) && substr($field['id'], 0, 9) == 'previous_' ) // don't save any fields storing previous data
                    {
                        continue;
                    }

                    if ( isset($field['type']) && $field['type'] != 'html' )
                    {
                        $field_value = '';
                        if ( isset($_POST[$format . '_' . $field['id']]) && !empty($_POST[$format . '_' . $field['id']]) )
                        {
                            $field_value = sanitize_text_field(wp_unslash($_POST[$format . '_' . $field['id']]));
                        }
                        if ( $field['id'] == 'property_node_options' || $field['id'] == 'property_field_options' )
                        {
                            $field_value = wp_unslash($field_value);
                        }
                        $export_options[$field['id']] = $field_value;

                        // Clear any old SHA1s if the API urls have changed
                        if ( 
                            $field['id'] == 'send_property_url' && 
                            isset($_POST[$format . '_previous_' . $field['id']]) && 
                            !empty(sanitize_text_field($_POST[$format . '_previous_' . $field['id']])) &&
                            sanitize_text_field(wp_unslash($_POST[$format . '_' . $field['id']])) != sanitize_text_field(wp_unslash($_POST[$format . '_previous_' . $field['id']]))
                        )
                        {
                            // we got a different URL. Clear the SHA1s
                            delete_post_meta_by_key( '_realtime_sha1_' . $export_id );
                            delete_post_meta_by_key( '_zoopla_sha1_' . $export_id );
                        }
                    }
                }
            }
        }

        $export_mappings = array();

        if ( isset($_POST['taxonomy_mapping']) && is_array($_POST['taxonomy_mapping']) && !empty($_POST['taxonomy_mapping']) )
        {
            foreach ( $_POST['taxonomy_mapping'] as $taxonomy => $mappings )
            {
                $taxonomy = sanitize_text_field(wp_unslash($taxonomy));

                $export_mappings[$taxonomy] = array();

                if ( is_array($mappings) && !empty($mappings) )
                {
                    foreach ( $mappings as $term_id => $crm_id )
                    {
                        if ( $crm_id != '' )
                        {
                            $export_mappings[$taxonomy][$term_id] = $crm_id;
                        }
                    }
                }
            }
        }

        $export_options['mappings'] = $export_mappings;

        $options['exports'][$export_id] = $export_options;

        update_option( 'realhomes_property_feed', $options );

        wp_redirect( admin_url( 'admin.php?page=realhomes-property-feed-export&rhpfsuccessmessage=' . urlencode( __( 'Export details saved', 'realhomes-property-feed' ) ) ) );
        die();
    }

    public function toggle_export_running_status()
    {
        if ( isset($_GET['action']) && in_array($_GET['action'], array("startexport", "pauseexport", "pushall")) && isset($_GET['export_id']) )
        {
            $export_id = !empty($_GET['export_id']) ? (int)$_GET['export_id'] : '';

            if ( empty($export_id) )
            {
                wp_redirect( admin_url( 'admin.php?page=realhomes-property-feed-export&rhpferrormessage=' . urlencode( __( 'No export passed', 'realhomes-property-feed' ) ) ) );
                die();
            }

            $options = get_option( 'realhomes_property_feed', array() );
            
            if ( !isset($options['exports'][$export_id]) )
            {
                wp_redirect( admin_url( 'admin.php?page=realhomes-property-feed-export&rhpferrormessage=' . urlencode( __( 'Export not found', 'realhomes-property-feed' ) ) ) );
                die();
            }

            switch ( sanitize_text_field(wp_unslash($_GET['action'])) )
            {
                case "startexport":
                {   
                    // Check one exports not already active if not using pro
                    if ( apply_filters( 'realhomes_property_feed_pro_active', false ) !== true ) 
                    {
                        foreach ( $options['exports'] as $export )
                        {
                            if ( ( !isset($export['deleted']) || ( isset($export['deleted']) && $export['deleted'] !== true ) ) && isset($export['running']) && $export['running'] === true )
                            {
                                wp_redirect( admin_url( 'admin.php?page=realhomes-property-feed-export&rhpferrormessage=' . urlencode(__( 'Maximum number of running exports reached. Upgrade to PRO if wanting to benfit from multiple exports and more', 'realhomes-property-feed' ) ) ) );
                                die();
                            }
                        }
                    }

                    $options['exports'][$export_id]['running'] = true;

                    update_option( 'realhomes_property_feed', $options );

                    wp_redirect( admin_url( 'admin.php?page=realhomes-property-feed-export&rhpfsuccessmessage=' . urlencode( __( 'Export started', 'realhomes-property-feed' ) ) ) );
                    die();

                    break;
                }
                case "pauseexport":
                {
                    $options['exports'][$export_id]['running'] = false;

                    update_option( 'realhomes_property_feed', $options );

                    wp_redirect( admin_url( 'admin.php?page=realhomes-property-feed-export&rhpfsuccessmessage=' . urlencode( __( 'Export paused', 'realhomes-property-feed' ) ) ) );
                    die();

                    break;
                }
                case "pushall":
                {
                    if ( ! isset( $_GET['_wpnonce'] ) || ! wp_verify_nonce( sanitize_text_field(wp_unslash($_GET['_wpnonce'])), 'push-all' ) )
                    {
                        wp_redirect( admin_url( 'admin.php?page=realhomes-property-feed-export&rhpferrormessage=' . urlencode( __( 'Nonce verification failed. Please try again', 'realhomes-property-feed' ) ) ) );
                        die();
                    }

                    if ( $options['exports'][$export_id]['running'] !== true )
                    {
                        wp_redirect( admin_url( 'admin.php?page=realhomes-property-feed-export&rhpferrormessage=' . urlencode( __( 'Export not active', 'realhomes-property-feed' ) ) ) );
                        die();
                    }

                    if ( $options['exports'][$export_id]['deleted'] === true )
                    {
                        wp_redirect( admin_url( 'admin.php?page=realhomes-property-feed-export&rhpferrormessage=' . urlencode( __( 'Exportdeletede', 'realhomes-property-feed' ) ) ) );
                        die();
                    }

                    $format = realhomes_property_feed_get_export_format( $options['exports'][$export_id]['format'] );

                    if ( $format['method'] != 'realtime' )
                    {
                        wp_redirect( admin_url( 'admin.php?page=realhomes-property-feed-export&rhpferrormessage=' . urlencode( __( 'Export not a real-time export', 'realhomes-property-feed' ) ) ) );
                        die();
                    }

                    do_action( 'realhomes_property_feed_push_all' );

                    wp_redirect( admin_url( 'admin.php?page=realhomes-property-feed-export&rhpfsuccessmessage=' . urlencode( __( 'All properties pushed', 'realhomes-property-feed' ) ) ) );
                    die();

                    break;
                }
            }
        }
    }

    public function delete_export()
    {
        if ( isset($_GET['action']) && $_GET['action'] == 'deleteexport' && isset($_GET['export_id']) )
        {
            if ( !isset($_GET['_wpnonce']) || !check_admin_referer('delete-export') )
            {
                wp_redirect( admin_url( 'admin.php?page=realhomes-property-feed-export&rhpferrormessage=' . urlencode( __( 'Security check failed', 'realhomes-property-feed' ) ) ) );
                die();
            }

            $export_id = !empty($_GET['export_id']) ? (int)$_GET['export_id'] : '';

            if ( empty($export_id) )
            {
                wp_redirect( admin_url( 'admin.php?page=realhomes-property-feed-export&rhpferrormessage=' . urlencode( __( 'No export passed', 'realhomes-property-feed' ) ) ) );
                die();
            }

            $options = get_option( 'realhomes_property_feed', array() );
            
            if ( !isset($options['exports'][$export_id]) )
            {
                wp_redirect( admin_url( 'admin.php?page=realhomes-property-feed-export&rhpferrormessage=' . urlencode( __( 'Export not found', 'realhomes-property-feed' ) ) ) );
                die();
            }

            $options['exports'][$export_id]['running'] = false;
            $options['exports'][$export_id]['deleted'] = true;

            update_option( 'realhomes_property_feed', $options );

            wp_redirect( admin_url( 'admin.php?page=realhomes-property-feed-export&rhpfsuccessmessage=' . urlencode( __( 'Export deleted successfully', 'realhomes-property-feed' ) ) ) );
            die();
        }
    }

    public function perform_field_mapping( $property, $post_id, $export_id )
    {
        $export_settings = realhomes_property_feed_get_export_settings_from_id( $export_id );

        if ( $export_settings === false )
        {
            return $property;
        }

        if ( !isset($export_settings['field_mapping_rules']) )
        {
            return $property;
        }

        if ( empty($export_settings['field_mapping_rules']) )
        {
            return $property;
        }

        global $post;

        $original_post = $post;

        $realhomes_fields = realhomes_property_feed_get_fields_for_field_mapping();

        foreach ( $export_settings['field_mapping_rules'] as $and_rules )
        {
            // realhomes_field
            // equal
            // field
            // result
            $rules_met = 0;
            foreach ( $and_rules['rules'] as $i => $rule )
            {
                if ( $rule['equal'] == '*' || get_post_meta( $post_id, $rule['realhomes_field'], TRUE ) == $rule['equal'] )
                {
                    ++$rules_met;
                }
            }

            if ( $rules_met == count($and_rules['rules']) )
            {
                $result = $and_rules['result'];
                
                preg_match_all('/{[^}]*}/', $and_rules['result'], $matches);
                if ( $matches !== FALSE && isset($matches[0]) && is_array($matches[0]) && !empty($matches[0]) )
                {
                    foreach ( $matches[0] as $match )
                    {
                        $field_name = str_replace(array("{", "}"), "", $match);

                        $value_to_check = get_post_meta( $post_id, $field_name, TRUE );
                        if ( empty($value_to_check) )
                        {
                            foreach ( $realhomes_fields as $realhomes_field_key => $realhomes_field )
                            {
                                if ( isset($realhomes_field['type']) && $realhomes_field['type'] == 'meta' )
                                {
                                    if ( isset($realhomes_field['label']) && $realhomes_field['label'] == $field_name )
                                    {
                                        $value_to_check = get_post_meta( $post_id, $realhomes_field_key, TRUE );
                                    }
                                }
                                if ( isset($realhomes_field['type']) && $realhomes_field['type'] == 'post_field' )
                                {
                                    if ( isset($realhomes_field['label']) && $realhomes_field['label'] == $field_name )
                                    {
                                        $temp_post = get_post($post_id, ARRAY_A);
                                        if ( isset($temp_post[$realhomes_field_key]) )
                                        {
                                            $value_to_check = $temp_post[$realhomes_field_key];
                                        }
                                    }
                                }
                            }
                        }
                        $result = str_replace($match, $value_to_check, $result);
                    }
                }

                if ( is_object($property) )
                {
                    // must be SimpleXML like Kyero
                    $property_nodes_to_update = $property->xpath($and_rules['field']);
                    if ( $property_nodes_to_update !== FALSE && !empty($property_nodes_to_update) )
                    {
                        if ( isset($property_nodes_to_update[0][0]) )
                        {
                            $property_nodes_to_update[0][0] = $result;
                        }
                    }
                }
                else
                {
                    $property[$and_rules['field']] = $result;
                }
            }
        }

        $post = $original_post;
        setup_postdata((int)$post_id);

        return $property;
    }
}

new RealHomes_Property_Feed_Export();