<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * RealHomes Property Feed Settings Functions
 */
class RealHomes_Property_Feed_Settings {

	public function __construct() {

        add_action( 'admin_init', array( $this, 'save_settings') );

	}

    public function save_settings()
    {
        if ( !isset($_POST['save_rhpf_settings']) )
        {
            return;
        }

        if ( !isset($_POST['_wpnonce']) || ( isset($_POST['_wpnonce']) && !wp_verify_nonce( sanitize_text_field(wp_unslash($_POST['_wpnonce'])), 'save-rhpf-settings' ) ) ) 
        {
            die( esc_html(__( "Failed security check", 'realhomes-property-feed' )) );
        }

        $options = get_option( 'realhomes_property_feed' , array() );
        if ( !is_array($options) ) { $options = array(); }

        if ( isset($_GET['page']) && sanitize_text_field($_GET['page']) == 'realhomes-property-feed-import' )
        {
            $new_options = array(
                'email_reports' => ( ( isset($_POST['email_reports']) && $_POST['email_reports'] == 'yes' ) ? true : false ),
                'email_reports_to' => ( ( isset($_POST['email_reports_to']) && sanitize_email($_POST['email_reports_to']) ) ? sanitize_email(wp_unslash($_POST['email_reports_to'])) : '' ),
                'remove_action' => ( ( isset($_POST['remove_action']) && in_array($_POST['remove_action'], array( '', 'nothing', 'remove_all_media', 'delete' )) ) ? sanitize_text_field(wp_unslash($_POST['remove_action'])) : '' ),
                'media_processing' => ( ( isset($_POST['media_processing']) && in_array($_POST['media_processing'], array( '', 'background' )) ) ? sanitize_text_field(wp_unslash($_POST['media_processing'])) : '' ),
            );
        }

        if ( isset($_GET['page']) && sanitize_text_field($_GET['page']) == 'realhomes-property-feed-export' )
        {
            $new_options = array(
                'sales_statuses' => ( ( isset($_POST['sales_statuses']) && !empty($_POST['sales_statuses']) ) ? rhpf_clean( $_POST['sales_statuses'] ) : array() ),
                'lettings_statuses' => ( ( isset($_POST['lettings_statuses']) && !empty($_POST['lettings_statuses']) ) ? rhpf_clean( $_POST['lettings_statuses'] ) : array() ),
                'property_selection' => ( ( isset($_POST['property_selection']) && in_array($_POST['property_selection'], array( '', 'individual', 'per_export' )) ) ? sanitize_text_field(wp_unslash($_POST['property_selection'])) : '' ),
            );
        }

        $options = array_merge( $options, $new_options );

        update_option( 'realhomes_property_feed', $options );

        wp_redirect( admin_url( 'admin.php?page=' . ( isset($_GET['page']) ? sanitize_text_field(wp_unslash($_GET['page'])) : 'realhomes-property-feed-import' ) . '&tab=settings&rhpfsuccessmessage=' . __( 'Settings saved', 'realhomes-property-feed' ) ) );
        die();
    }
}

new RealHomes_Property_Feed_Settings();