<?php

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

function realhomes_property_feed_get_import_formats()
{
    $curl_warning = !function_exists('curl_version') ? __( 'cURL must be enabled in order to use this format', 'realhomes-property-feed' ) : '';
    $simplexml_warning = !class_exists('SimpleXMLElement') ? __( 'SimpleXML must be enabled in order to use this format', 'realhomes-property-feed' ) : '';

    $uploads_dir = wp_upload_dir();
    if( $uploads_dir['error'] === FALSE )
    {
        $uploads_dir = $uploads_dir['basedir'] . '/realhomes_property_feed_import/';
    }

    $formats = array(
        '10ninety' => array(
            'name' => __( '10ninety', 'realhomes-property-feed' ),
            'fields' => array(
                array(
                    'id' => 'xml_url',
                    'label' => __( 'XML URL', 'realhomes-property-feed' ),
                    'type' => 'text',
                    'placeholder' => 'https://',
                )
            ),
            'address_fields' => array( 'ADDRESS_2', 'ADDRESS_3', 'TOWN', 'ADDRESS_4', 'COUNTY' ),
            'taxonomy_values' => array(
                'sales_status' => array(
                    '0' => 'Available',
                    '1' => 'SSTC',
                    '2' => 'SSTCM (Scotland only)',
                    '3' => 'Under Offer',
                    '6' => 'Sold',
                ),
                'lettings_status' => array(
                    '0' => 'Available',
                    '4' => 'Reserved',
                    '5' => 'Let Agreed',
                    '7' => 'Let',
                ),
                'property_type' => array(
                    '0' => 'Not Specified',
                    '1' => 'Terraced',
                    '2' => 'End of Terrace',
                    '3' => 'Semi-Detached',
                    '4' => 'Detached',
                    '5' => 'Mews',
                    '6' => 'Cluster House',
                    '7' => 'Ground Flat',
                    '8' => 'Flat',
                    '9' => 'Studio',
                    '10' => 'Ground Maisonette',
                    '11' => 'Maisonette',
                    '12' => 'Bungalow',
                    '13' => 'Terraced Bungalow',
                    '14' => 'Semi-Detached Bungalow',
                    '15' => 'Detached Bungalow',
                    '16' => 'Mobile Home',
                    '17' => 'Hotel',
                    '18' => 'Guest House',
                    '20' => 'Land',
                    '21' => 'Link Detached House',
                    '22' => 'Town House',
                    '23' => 'Cottage',
                    '24' => 'Chalet',
                    '27' => 'Villa',
                    '28' => 'Apartment',
                    '29' => 'Penthouse',
                    '30' => 'Finca',
                    '43' => 'Barn Conversion',
                    '44' => 'Serviced Apartments',
                    '45' => 'Parking',
                    '46' => 'Sheltered Housing',
                    '47' => 'Retirement Property',
                    '48' => 'House Share',
                    '49' => 'Flat Share',
                    '51' => 'Garages',
                    '52' => 'Farm House',
                    '53' => 'Equestrian',
                    '56' => 'Duplex',
                    '59' => 'Triplex',
                    '68' => 'Barn',
                    '95' => 'Village House',
                    '107' => 'Farm Land',
                    '110' => 'Plot',
                    '113' => 'Country House',
                    '116' => 'Stone House',
                    '117' => 'Caravan',
                    '118' => 'Lodge',
                    '120' => 'Manor House',
                    '121' => 'Stately Home',
                    '125' => 'Off-Plan',
                    '128' => 'Semi-detached Villa',
                    '131' => 'Detached Villa',
                    '142' => 'Hotel Room',
                    '143' => 'Block of Apartments',
                    '144' => 'Private Halls',
                    '253' => 'Commercial Property',
                )
            ),
            'contact_information_fields' => array(
                'BRANCH_ID',
            ),
            'help_url' => 'https://realhomespropertyfeed.com/documentation/managing-imports/formats/10ninety/',
            'warnings' => array_filter( array( $simplexml_warning ) ),
            'background_mode' => true,
        ),
        'street' => array(
            'name' => __( 'Street', 'realhomes-property-feed' ),
            'fields' => array(
                array(
                    'id' => 'api_key',
                    'label' => __( 'API Key', 'realhomes-property-feed' ),
                    'type' => 'text',
                ),
                array(
                    'id' => 'base_url',
                    'label' => __( 'API Base URL', 'realhomes-property-feed' ),
                    'type' => 'text',
                    'default' => 'https://street.co.uk',
                ),
                array(
                    'id' => 'sales_statuses',
                    'label' => __( 'Sales Status(es) To Import', 'realhomes-property-feed' ),
                    'type' => 'multiselect',
                    'options' => array( 
                        'for_sale' => 'for_sale', 
                        'for_sale_and_to_let' => 'for_sale_and_to_let', 
                        'under_offer' => 'under_offer', 
                        'sold_stc' => 'sold_stc', 
                        'exchanged' => 'exchanged', 
                        'completed'  => 'completed' 
                    ),
                    'default' => apply_filters( 'realhomes_property_feed_street_sales_statuses', array( 'for_sale', 'for_sale_and_to_let', 'under_offer', 'sold_stc' ) ),
                    'tooltip' => 'One or more must be selected. Ctrl/Cmd + Click to select multiple',
                ),
                array(
                    'id' => 'lettings_statuses',
                    'label' => __( 'Lettings Status(es) To Import', 'realhomes-property-feed' ),
                    'type' => 'multiselect',
                    'options' => array( 
                        'to_let' => 'to_let', 
                        'for_sale_and_to_let' => 'for_sale_and_to_let', 
                        'let_agreed' => 'let_agreed', 
                        'let' => 'let' 
                    ),
                    'default' => apply_filters( 'realhomes_property_feed_street_lettings_statuses', array( 'to_let', 'for_sale_and_to_let', 'let_agreed' ) ),
                    'tooltip' => 'One or more must be selected. Ctrl/Cmd + Click to select multiple',
                )
            ),
            'address_fields' => array( 'town', 'line_2', 'line_3' ),
            'taxonomy_values' => array(
                'sales_status' => array(
                    'For Sale' => 'For Sale',
                    'Under Offer' => 'Under Offer',
                    'Sold STC' => 'Sold STC',
                    'For Sale and To Let' => 'For Sale and To Let',
                ),
                'lettings_status' => array(
                    'To Let' => 'To Let',
                    'Let Agreed' => 'Let Agreed',
                    'For Sale and To Let' => 'For Sale and To Let',
                ),
                'property_type' => array(
                    'Detached House' => 'Detached House',
                    'Semi-Detached House' => 'Semi-Detached House',
                    'Terraced House' => 'Terraced House',
                    'Bungalow' => 'Bungalow',
                    'Flat / Apartment' => 'Flat / Apartment',
                    'Other' => 'Other',
                )
            ),
            'contact_information_fields' => array(
                'branch_uuid',
            ),
            'help_url' => 'https://realhomespropertyfeed.com/documentation/managing-imports/formats/street/'
        ),
    );

    $formats = apply_filters( 'realhomes_property_feed_import_formats', $formats );

    uasort($formats, 'realhomes_property_feed_compare_by_name');

    return $formats;
}

function realhomes_property_feed_compare_by_name($a, $b) 
{
    return strcasecmp($a['name'], $b['name']);
}

function realhomes_property_feed_get_import_format( $key )
{
    $formats = realhomes_property_feed_get_import_formats();
    
    return isset($formats[$key]) ? $formats[$key] : false;
}

function realhomes_property_feed_get_format_from_import_id( $import_id )
{
    $formats = realhomes_property_feed_get_import_formats();

    $options = get_option( 'realhomes_property_feed' , array() );
    $imports = ( isset($options['imports']) && is_array($options['imports']) && !empty($options['imports']) ) ? $options['imports'] : array();

    if ( isset($imports[$import_id]) )
    {
        $format = $imports[$import_id]['format'];

        return realhomes_property_feed_get_import_format( $format );
    }
    
    return false;
}

function realhomes_property_feed_get_export_formats()
{
    $curl_warning = !function_exists('curl_version') ? __( 'cURL must be enabled in order to use this format', 'realhomes-property-feed' ) : '';
    $simplexml_warning = !class_exists('SimpleXMLElement') ? __( 'SimpleXML must be enabled in order to use this format', 'realhomes-property-feed' ) : '';
    // FTP warning?
    // zipArchive warning?

    $branch_mapping_options = array();

    $args = array(
        'post_type' => 'property',
        'fields' => 'ids',
        'meta_query' => array(
            array(
                'key' => 'REAL_HOMES_agent_display_option',
                'value' => 'author_info',
            )
        ),
    );

    $property_query = new WP_Query( $args );

    if ( $property_query->have_posts() )
    {
        $branch_mapping_options['author_info'] = array();

        $users = get_users( array( 'orderby' => 'name' ) );
        foreach ( $users as $user ) 
        {
            $branch_mapping_options['author_info'][$user->ID] = $user->display_name;
        }
    }

    $args = array(
        'post_type' => 'property',
        'fields' => 'ids',
        'meta_query' => array(
            array(
                'key' => 'REAL_HOMES_agent_display_option',
                'value' => 'agent_info',
            )
        ),
    );

    $property_query = new WP_Query( $args );

    if ( $property_query->have_posts() )
    {
        $args = array(
            'post_type' => 'agent',
            'nopaging' => true
        );

        $agent_query = new WP_Query( $args );

        if ( $agent_query->have_posts() )
        {
            $branch_mapping_options['agent_info'] = array();

            while ( $agent_query->have_posts() )
            {
                $agent_query->the_post();

                $branch_mapping_options['agent_info'][get_the_ID()] = get_the_title();
            }
        }
    }

    $branch_mapping_fields = array(
        array(
            'type' => 'html',
            'html' => '<p style="font-size:1.1em"><strong>' . __( 'Branch Codes', 'realhomes-property-feed' ) . '</strong></p>',
        )
    );

    if ( !empty($branch_mapping_options) )
    {
        foreach ( $branch_mapping_options as $type => $values )
        {
            foreach ( $values as $id => $name )
            {
                $branch_mapping_fields[] = array(
                    'type' => 'text',
                    'id' => 'branch_code_' . $type . '_' . $id . '_sales',
                    'label' => $name . ' (' . ucwords(str_replace("_info", "", $type)) . ') - ' . __( 'Sales', 'realhomes-property-feed' ),
                );
                $branch_mapping_fields[] = array(
                    'type' => 'text',
                    'id' => 'branch_code_' . $type . '_' . $id . '_lettings',
                    'label' => $name . ' (' . ucwords(str_replace("_info", "", $type)) . ') - ' . __( 'Lettings', 'realhomes-property-feed' ),
                );
            }
        }
    }

    $formats = array(
        'blm' => apply_filters( 'realhomes_property_feed_export_format_options_blm', array(
            'name' => __( 'BLM', 'realhomes-property-feed' ),
            'method' => 'cron', // cron / realtime / url
            'fields' => array_merge(array(
                array(
                    'type' => 'html',
                    'html' => '<p style="font-size:1.1em"><strong>' . __( 'FTP Details', 'realhomes-property-feed' ) . '</strong></p>',
                ),
                array(
                    'id' => 'ftp_host',
                    'label' => __( 'FTP Host', 'realhomes-property-feed' ),
                    'type' => 'text',
                ),
                array(
                    'id' => 'ftp_user',
                    'label' => __( 'FTP Username', 'realhomes-property-feed' ),
                    'type' => 'text',
                ),
                array(
                    'id' => 'ftp_pass',
                    'label' => __( 'FTP Password', 'realhomes-property-feed' ),
                    'type' => 'text',
                ),
                array(
                    'id' => 'ftp_dir',
                    'label' => __( 'FTP Directory', 'realhomes-property-feed' ),
                    'type' => 'text',
                ),
                array(
                    'id' => 'ftp_passive',
                    'label' => __( 'Use FTP Passive Mode', 'realhomes-property-feed' ),
                    'type' => 'checkbox',
                )
            ), $branch_mapping_fields, array(
                array(
                    'type' => 'html',
                    'html' => '<p style="font-size:1.1em"><strong>' . __( 'Advanced Settings', 'realhomes-property-feed' ) . '</strong></p>',
                ),
                /*array(
                    'id' => 'overseas',
                    'label' => __( 'Use Overseas Format', 'realhomes-property-feed' ),
                    'type' => 'checkbox',
                    'tooltip' => __( 'If selected, version 3i of the BLM format will be used<br>For overseas to work you must have the RealHomes Country taxonomy enabled and a country selected that\'s not anything other than UK, United Kingdom, GB, Britain, Great Britain, England, Scotland, Wales or Ireland', 'realhomes-property-feed' )
                ),*/
                array(
                    'id' => 'media_sent_as',
                    'label' => __( 'Media Sent as', 'realhomes-property-feed' ),
                    'type' => 'select',
                    'options' => array(
                        'urls' => 'URLs',
                        'files' => 'Files',
                    ),
                    'tooltip' => 'URLs: Full URL to any media will be sent and the third party will need to download the files<br>Files: The physical media files will be sent to the portal'
                ),
                array(
                    'id' => 'incremental',
                    'label' => __( 'Incremental', 'realhomes-property-feed' ),
                    'type' => 'checkbox',
                    'tooltip' => 'Only applicable when \'Files\' is selected above. If this setting is checked only new or changed media will be included in the upload, thus preventing every media file to be sent every time an export runs. We advise leaving this unticked whilst the feed is being setup. Only tick this once it\'s all up and running'
                ),
                array(
                    'id' => 'compressed',
                    'label' => __( 'Compress Files Into ZIP', 'realhomes-property-feed' ),
                    'type' => 'checkbox',
                    'tooltip' => __( 'Compress everything into a single ZIP file and upload that one file', 'realhomes-property-feed' ),
                )
            )),
            'field_mapping_fields' => array(
                'ADDRESS_1' => 'ADDRESS_1',
                'ADDRESS_2' => 'ADDRESS_2 (non-overseas only)',
                'ADDRESS_3' => 'ADDRESS_3 (non-overseas only)',
                'TOWN' => 'TOWN (non-overseas only)',
                'POSTCODE1' => 'POSTCODE1 (non-overseas only)',
                'POSTCODE2' => 'POSTCODE2 (non-overseas only)',
                'HOUSE_NAME_NUMBER' => 'HOUSE_NAME_NUMBER (overseas only)',
                'STREET_NAME' => 'STREET_NAME (overseas only)',
                'OS_TOWN_CITY' => 'OS_TOWN_CITY (overseas only)',
                'OS_REGION' => 'OS_REGION (overseas only)',
                'ZIPCODE' => 'ZIPCODE (overseas only)',
                'COUNTRY_CODE' => 'COUNTRY_CODE (overseas only)',
                'EXACT_LATITUDE' => 'EXACT_LATITUDE (overseas only)',
                'EXACT_LONGITUDE' => 'EXACT_LONGITUDE (overseas only)',
                'FEATURE1' => 'FEATURE1',
                'FEATURE2' => 'FEATURE2',
                'FEATURE3' => 'FEATURE3',
                'FEATURE4' => 'FEATURE4',
                'FEATURE5' => 'FEATURE5',
                'FEATURE6' => 'FEATURE6',
                'FEATURE7' => 'FEATURE7',
                'FEATURE8' => 'FEATURE8',
                'FEATURE9' => 'FEATURE9',
                'FEATURE10' => 'FEATURE10',
                'SUMMARY' => 'SUMMARY',
                'DESCRIPTION' => 'DESCRIPTION',
                'STATUS_ID' => 'STATUS_ID',
                'BEDROOMS' => 'BEDROOMS',
                'BATHROOMS' => 'BATHROOMS',
                'LIVING_ROOMS' => 'LIVING_ROOMS',
                'PRICE' => 'PRICE',
                'PRICE_QUALIFIER' => 'PRICE_QUALIFIER',
                'PROP_SUB_ID' => 'PROP_SUB_ID',
                'CREATE_DATE' => 'CREATE_DATE',
                'UPDATE_DATE' => 'UPDATE_DATE',
                'DISPLAY_ADDRESS' => 'DISPLAY_ADDRESS',
                'PUBLISHED_FLAG' => 'PUBLISHED_FLAG',
                'LET_DATE_AVAILABLE' => 'LET_DATE_AVAILABLE',
                'LET_BOND' => 'LET_BOND',
                'ADMINISTRATION_FEE' => 'ADMINISTRATION_FEE',
                'LET_TYPE_ID' => 'LET_TYPE_ID',
                'LET_FURN_ID' => 'LET_FURN_ID',
                'LET_RENT_FREQUENCY' => 'LET_RENT_FREQUENCY',
                'TENURE_TYPE_ID' => 'TENURE_TYPE_ID',
                'COUNCIL_TAX_BAND' => 'COUNCIL_TAX_BAND',
                'SHARED_OWNERSHIP' => 'SHARED_OWNERSHIP',
                'SHARED_OWNERSHIP_PERCENTAGE' => 'SHARED_OWNERSHIP_PERCENTAGE',
                'ANNUAL_GROUND_RENT' => 'ANNUAL_GROUND_RENT',
                'GROUND_RENT_REVIEW_PERIOD_YEARS' => 'GROUND_RENT_REVIEW_PERIOD_YEARS',
                'ANNUAL_SERVICE_CHARGE' => 'ANNUAL_SERVICE_CHARGE',
                'TENURE_UNEXPIRED_YEARS' => 'TENURE_UNEXPIRED_YEARS',
                'TRANS_TYPE_ID' => 'TRANS_TYPE_ID',
            ),
            'taxonomy_values' => array(
                'status' => array(
                    '0' => 'Available',
                    '1' => 'SSTC',
                    '2' => 'SSTCM (Scotland only)',
                    '3' => 'Under Offer',
                    '4' => 'Reserved',
                    '5' => 'Let Agreed',
                    '6' => 'Sold',
                    '7' => 'Let',
                ),
                'property_type' => array(
                    '0' => 'Not Specified',
                    '1' => 'Terraced',
                    '2' => 'End of Terrace',
                    '3' => 'Semi-Detached ',
                    '4' => 'Detached',
                    '5' => 'Mews',
                    '6' => 'Cluster House',
                    '7' => 'Ground Flat',
                    '8' => 'Flat',
                    '9' => 'Studio',
                    '10' => 'Ground Maisonette',
                    '11' => 'Maisonette',
                    '12' => 'Bungalow',
                    '13' => 'Terraced Bungalow',
                    '14' => 'Semi-Detached Bungalow',
                    '15' => 'Detached Bungalow',
                    '16' => 'Mobile Home',
                    '17' => 'Hotel',
                    '18' => 'Guest House',
                    '20' => 'Land',
                    '21' => 'Link Detached House',
                    '22' => 'Town House',
                    '23' => 'Cottage',
                    '24' => 'Chalet',
                    '27' => 'Villa',
                    '28' => 'Apartment',
                    '29' => 'Penthouse',
                    '30' => 'Finca',
                    '43' => 'Barn Conversion',
                    '44' => 'Serviced Apartments',
                    '45' => 'Parking',
                    '46' => 'Sheltered Housing',
                    '47' => 'Retirement Property',
                    '48' => 'House Share',
                    '49' => 'Flat Share',
                    '51' => 'Garages',
                    '52' => 'Farm House',
                    '53' => 'Equestrian',
                    '56' => 'Duplex',
                    '59' => 'Triplex',
                    '68' => 'Barn',
                    '95' => 'Village House',
                    '107' => 'Farm Land',
                    '110' => 'Plot',
                    '113' => 'Country House',
                    '116' => 'Stone House',
                    '117' => 'Caravan',
                    '118' => 'Lodge',
                    '120' => 'Manor House',
                    '121' => 'Stately Home',
                    '125' => 'Off-Plan',
                    '128' => 'Semi-detached Villa',
                    '131' => 'Detached Villa',
                    '142' => 'Hotel Room',
                    '143' => 'Block of Apartments',
                    '144' => 'Private Halls',
                    '253' => 'Commercial Property',
                ),
                'price_qualifier' => array(
                    '1' => 'POA',
                    '2' => 'Guide Price',
                    '3' => 'Fixed Price',
                    '4' => 'Offers in Excess of',
                    '5' => 'OIRO',
                    '6' => 'Sale by Tender',
                    '7' => 'From',
                    '9' => 'Shared Ownership',
                    '10' => 'Offers Over',
                    '11' => 'Part Buy Part Rent',
                    '12' => 'Shared Equity',
                ),
            ),
            'help_url' => 'https://realhomespropertyfeed.com/documentation/managing-exports/formats/blm/',
            'warnings' => array(), // maybe FTP warning? maybe ZipArchive warning
        ) ),
    );

    $formats = apply_filters( 'realhomes_property_feed_export_formats', $formats );

    return $formats;
}

function realhomes_property_feed_get_export_format( $key )
{
    $formats = realhomes_property_feed_get_export_formats();
    
    return isset($formats[$key]) ? $formats[$key] : false;
}

function realhomes_property_feed_get_format_from_export_id( $export_id )
{
    $formats = realhomes_property_feed_get_export_formats();

    $options = get_option( 'realhomes_property_feed' , array() );
    $exports = ( isset($options['exports']) && is_array($options['exports']) && !empty($options['exports']) ) ? $options['exports'] : array();

    if ( isset($exports[$export_id]) )
    {
        $format = $exports[$export_id]['format'];

        return realhomes_property_feed_get_export_format( $format );
    }
    
    return false;
}