<?php

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

function realhomes_property_feed_get_import_settings_from_id( $import_id )
{
    $options = get_option( 'realhomes_property_feed' , array() );
    $imports = ( isset($options['imports']) && is_array($options['imports']) && !empty($options['imports']) ) ? $options['imports'] : array();

    if ( isset($imports[$import_id]) )
    {
        return $imports[$import_id];
    }

    return false;
}

function realhomes_property_feed_convert_old_field_mapping_to_new( $field_mapping_rules )
{
    $old_style = false;

    foreach ( $field_mapping_rules as $rule )
    {
        if ( isset($rule['field']) )
        {
            $old_style = true;
            break;
        }
    }

    if ( $old_style === true )
    {
        // need to convert
        $new_field_mapping_rules = array();
        foreach ( $field_mapping_rules as $rule )
        {
            if ( $rule['result'] == '{field_value}' )
            {
                $rule['result'] = '{' . $rule['field'] . '}';
            }

            $new_field_mapping_rules[] = array(
                'realhomes_field' => $rule['realhomes_field'],
                'result' => $rule['result'],
                'rules' => array(
                    array(
                        'field' => $rule['field'],
                        'equal' => $rule['equal']
                    )
                )
            );
        }
        $field_mapping_rules = $new_field_mapping_rules;
    }

    return $field_mapping_rules;
}

function realhomes_property_feed_get_fields_for_field_mapping()
{
    $realhomes_fields = array(
        // Post Fields
        'post_title' => array( 'type' => 'post_field', 'label' => __( 'Post Title', 'realhomes-property-feed' ) ),
        'post_excerpt' => array( 'type' => 'post_field', 'label' => __( 'Post Excerpt', 'realhomes-property-feed' ) ),
        'post_content' => array( 'type' => 'post_field', 'label' => __( 'Post Content', 'realhomes-property-feed' ) ),
        'post_status' => array( 'type' => 'post_field', 'label' => __( 'Post Status', 'realhomes-property-feed' ), 'options' => array( 'publish' => 'Publish', 'private' => 'Private', 'draft' => 'Draft') ),
        // RealHomes Fields
        'REAL_HOMES_property_old_price' => array( 'type' => 'meta', 'label' => __( 'Old Price', 'realhomes-property-feed' ) ),
        'REAL_HOMES_property_price_prefix' => array( 'type' => 'meta', 'label' => __( 'Price Prefix', 'realhomes-property-feed' ) ),
        'REAL_HOMES_property_price_postfix' => array( 'type' => 'meta', 'label' => __( 'Price Postfix', 'realhomes-property-feed' ) ),
        'REAL_HOMES_property_size' => array( 'type' => 'meta', 'label' => __( 'Area Size', 'realhomes-property-feed' ) ),
        'REAL_HOMES_property_size_postfix' => array( 'type' => 'meta', 'label' => __( 'Area Size Postfix', 'realhomes-property-feed' ) ),
        'REAL_HOMES_property_bedrooms' => array( 'type' => 'meta', 'label' => __( 'Bedrooms', 'realhomes-property-feed' ) ),
        'REAL_HOMES_property_bathrooms' => array( 'type' => 'meta', 'label' => __( 'Bathrooms', 'realhomes-property-feed' ) ),
        'REAL_HOMES_property_garage' => array( 'type' => 'meta','label' =>  __( 'Garages or Parking Spaces', 'realhomes-property-feed' ) ),
        'REAL_HOMES_property_year_built' => array( 'type' => 'meta', 'label' => __( 'Year Built', 'realhomes-property-feed' ) ),
        'REAL_HOMES_property_id' => array( 'type' => 'meta', 'label' => __( 'Property ID / Reference Number', 'realhomes-property-feed' ) ),
        'REAL_HOMES_property_address' => array( 'type' => 'meta', 'label' => __( 'Property Address', 'realhomes-property-feed' ) ),
        'REAL_HOMES_property_map' => array( 'type' => 'meta', 'label' => __( 'Hide Map', 'realhomes-property-feed' ), 'options' => array( 0 => 0, 1 => 1) ),
        //'realhomes_geolocation_lat' => array( 'type' => 'meta', 'label' => __( 'Latitude', 'realhomes-property-feed' ) ),
        //'realhomes_geolocation_long' => array( 'type' => 'meta', 'label' => __( 'Longitude', 'realhomes-property-feed' ) ),
        'REAL_HOMES_property_location' => array( 'type' => 'meta', 'label' => __( 'Location', 'realhomes-property-feed' ) . ' (format: lat,lng,zoom)' ),
        'REAL_HOMES_featured' => array( 'type' => 'meta', 'label' => __( 'Featured', 'realhomes-property-feed' ), 'options' => array( 0 => 0, 1 => 1) ),
        //'fave_property_disclaimer' => array( 'type' => 'meta', 'label' => __( 'Disclaimer', 'realhomes-property-feed' ) ),
        //'inspiry_video_group' => array( 'type' => 'meta', 'label' => __( 'Video URL', 'realhomes-property-feed' ) ),
        'REAL_HOMES_360_virtual_tour' => array( 'type' => 'meta', 'label' => __( '360° Virtual Tour', 'realhomes-property-feed' ) ),
        'REAL_HOMES_energy_class' => array( 'type' => 'meta', 'label' => __( 'Energy Class', 'realhomes-property-feed' ) ),
        'REAL_HOMES_energy_performance' => array( 'type' => 'meta', 'label' => __( 'Energy Performance', 'realhomes-property-feed' ) ),
        'REAL_HOMES_epc_current_rating' => array( 'type' => 'meta', 'label' => __( 'EPC Current Rating', 'realhomes-property-feed' ) ),
        'REAL_HOMES_epc_potential_rating' => array( 'type' => 'meta', 'label' => __( 'EPC Potential Rating', 'realhomes-property-feed' ) ),
        'REAL_HOMES_property_lot_size' => array( 'type' => 'meta', 'label' => __( 'Lot Size', 'realhomes-property-feed' ) ),
        'REAL_HOMES_property_lot_size_postfix' => array( 'type' => 'meta', 'label' => __( 'Lot Size Postfix', 'realhomes-property-feed' ) ),
        'REAL_HOMES_property_price' => array( 'type' => 'meta', 'label' => __( 'Sale or Rent Price', 'realhomes-property-feed' ) ),
    );
    
    // Contact agent related fields
    $fave_agent_display_options = array(
        'author_info' => __( 'Author / WordPress User', 'realhomes-property-feed' )
    );

    $fave_agent_display_options['agent_info'] = __( 'RealHomes Agent', 'realhomes-property-feed' );
    
    $fave_agent_display_options['none'] = __( 'Do Not Display', 'realhomes-property-feed' );

    $realhomes_fields['REAL_HOMES_agent_display_option'] = array( 'type' => 'meta', 'label' => __( 'Agent Display Option', 'realhomes-property-feed' ), 'options' => $fave_agent_display_options );

    // user/agent fields
    $wp_users = array();

    $users = get_users( array( 'orderby' => 'name' ) );
    foreach ( $users as $user ) 
    {
        $wp_users[$user->ID] = $user->display_name;
    }

    $realhomes_fields['post_author'] = array( 'type' => 'post_field', 'label' => __( 'Author / WordPress User', 'realhomes-property-feed' ), 'options' => $wp_users );

    $realhomes_agents = array();

    $realhomes_agents['auto'] = __( 'Automatically match based on name', 'realhomes-property-feed' );

    $args = array(
        'post_type' => '_agent',
        'nopaging' => true
    );

    $agent_query = new WP_Query( $args );

    if ( $agent_query->have_posts() )
    {
        while ( $agent_query->have_posts() )
        {
            $agent_query->the_post();

            $realhomes_agents[get_the_ID()] = get_the_title();
        }
    }
    wp_reset_postdata();

    $realhomes_fields['agents'] = array( 'type' => 'meta', 'label' => __( 'Agent Name', 'realhomes-property-feed' ), 'options' => $realhomes_agents );

    // add any fields from field builder
    $additional_fields = Run_Property_Additional_Fields()->get_additional_fields();

    if ( ! empty( $additional_fields ) ) 
    {
        foreach ( $additional_fields as $field ) 
        {
            $realhomes_fields[$field['field_key']] = array( 'type' => 'meta', 'label' => $field['field_name'], 'custom_field' => true, 'field_type' => $field['field_type'] );
        }
    }

    $taxonomies = array(
        'property-type' => array( 'type' => 'taxonomy', 'label' => __( 'Property Type', 'realhomes-property-feed' ) ),
        'property-status' => array( 'type' => 'taxonomy', 'label' => __( 'Property Status', 'realhomes-property-feed' ) ),
        'property-feature' => array( 'type' => 'taxonomy', 'label' => __( 'Property Features', 'realhomes-property-feed' ), 'delimited' => true ),
    );

    for ( $i = 0; $i < apply_filters( 'realhomes_property_feed_field_mapping_feature_count', 10 ); ++$i )
    {
        $taxonomies['property-feature[' . $i . ']'] = array( 'type' => 'taxonomy', 'label' => __( 'Property Feature', 'realhomes-property-feed' ) . ' ' . ( $i + 1 ) );
    }

    $taxonomies['property-city'] = array( 'type' => 'taxonomy', 'label' => __( 'Property Location', 'realhomes-property-feed' ) );

    $realhomes_fields = array_merge($realhomes_fields, $taxonomies);

    $realhomes_fields = apply_filters( 'realhomes_property_feed_field_mapping_realhomes_fields', $realhomes_fields );

    $realhomes_fields = realhomes_property_feed_array_msort( $realhomes_fields, array( 'label' => SORT_ASC ) );

    return $realhomes_fields;
}

function rhpf_determine_number_separators($number) 
{
    $decimalSeparator = '';
    $thousandSeparator = '';

    // Count occurrences
    $commaCount = substr_count($number, ',');
    $periodCount = substr_count($number, '.');

    // Logic to determine separators
    if ($commaCount > 0 && $periodCount > 0) {
        // Both symbols are present, determine based on position
        if (strrpos($number, ',') > strrpos($number, '.')) {
            $decimalSeparator = ',';
            $thousandSeparator = '.';
        } else {
            $decimalSeparator = '.';
            $thousandSeparator = ',';
        }
    } elseif ($commaCount > 0) {
        // Only commas are present
        if (($commaCount == 1 && strlen($number) - strrpos($number, ',') > 3) || ($commaCount > 1 && rhpf_is_thousands_grouping($number, ','))) {
            // Single comma or valid thousands grouping
            $thousandSeparator = ',';
            $decimalSeparator = '.';
        } else {
            // Comma likely used as decimal separator
            $decimalSeparator = ',';
            $thousandSeparator = '.';
        }
    } elseif ($periodCount > 0) {
        // Only periods are present
        if (($periodCount == 1 && strlen($number) - strrpos($number, '.') > 3) || ($periodCount > 1 && rhpf_is_thousands_grouping($number, '.'))) {
            // Single period or valid thousands grouping
            $thousandSeparator = '.';
            $decimalSeparator = ',';
        } else {
            // Period likely used as decimal separator
            $decimalSeparator = '.';
            $thousandSeparator = ',';
        }
    } else {
        // No separators found, default to common usage
        $decimalSeparator = '.';
        $thousandSeparator = ',';
    }

    return ['decimal' => $decimalSeparator, 'thousand' => $thousandSeparator];
}

// Helper function to check for consistent thousands grouping
function rhpf_is_thousands_grouping($number, $separator) {
    $parts = explode($separator, $number);

    // Allow the first part to have fewer than 3 digits
    foreach (array_slice($parts, 1, -1) as $part) 
    {
        if (strlen($part) !== 3) return false;
    }

    return true;
}

function rhpf_get_import_object_from_format($format, $instance_id, $import_id)
{
    switch ($format)
    {
        case "10ninety":
        {
            // includes
            require_once dirname( __FILE__ ) . '/import-formats/class-realhomes-property-feed-format-10ninety.php';

            $import_object = new Realhomes_Property_Feed_Format_10ninety( $instance_id, $import_id );

            break;
        }
        case "street":
        {
            // includes
            require_once dirname( __FILE__ ) . '/import-formats/class-realhomes-property-feed-format-street.php';

            $import_object = new Realhomes_Property_Feed_Format_Street( $instance_id, $import_id );

            break;
        }
        default:
        {
            $import_object = apply_filters( 'realhomes_property_feed_import_object', null, $instance_id, $import_id );
        }
    }

    return $import_object;
}