<?php
/**
 * Plugin Name: RealHomes Property Feed
 * Plugin Uri: https://realhomespropertyfeed.com
 * Description: Automatically import properties to RealHomes from estate agency CRMs and export to portals
 * Version: 1.0.0
 * Author: PropertyHive
 * Author URI: https://wp-property-hive.com
 * License: GPLv3
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'RealHomes_Property_Feed' ) ) :

final class RealHomes_Property_Feed {

    /**
     * @var string
     */
    public $version = '1.0.0';

    /**
     * @var RealHomes Property Feed The single instance of the class
     */
    protected static $_instance = null;
    
    /**
     * Main RealHomes Property Feed Instance
     *
     * Ensures only one instance of RealHomes Property Feed is loaded or can be loaded.
     *
     * @static
     * @return RealHomes Property Feed - Main instance
     */
    public static function instance() 
    {
        if ( is_null( self::$_instance ) ) 
        {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    /**
     * Constructor.
     */
    public function __construct() {

        // Define constants
        $this->define_constants();

        // Include required files
        $this->includes();

    }

    /**
     * Define RealHomes_Property_Feed Constants
     */
    private function define_constants() 
    {
        define( 'REALHOMES_PROPERTY_FEED_PLUGIN_FILE', __FILE__ );
        define( 'REALHOMES_PROPERTY_FEED_VERSION', $this->version );
    }

    private function includes()
    {
        include_once( 'includes/class-realhomes-property-feed-install.php' );
        include_once( 'includes/class-realhomes-property-feed-cron.php' );
        include_once( 'includes/class-realhomes-property-feed-redirect.php' );
        include_once( 'includes/class-realhomes-property-feed-license.php' );
        include_once( 'includes/class-realhomes-property-feed-import.php' );
        include_once( 'includes/class-realhomes-property-feed-export.php' );
        include_once( 'includes/class-realhomes-property-feed-settings.php' );
        include_once( 'includes/class-realhomes-property-feed-ajax.php' );
        //include_once( 'includes/class-realhomes-property-feed-wpml.php' );

        include_once( 'includes/import-functions.php' );
        include_once( 'includes/export-functions.php' );
        include_once( 'includes/format-functions.php' );
        include_once( 'includes/frequency-functions.php' );
        include_once( 'includes/xml-functions.php' );
        include_once( 'includes/array-functions.php' );
        include_once( 'includes/country-functions.php' );

        include_once( 'includes/class-realhomes-property-feed-process.php' );

        if ( is_admin() ) 
        {
            include_once( 'includes/class-realhomes-property-feed-admin.php' );
        }
    }

    
}

endif;

/**
 * Returns the main instance of RealHomes_Property_Feed to prevent the need to use globals.
 *
 * @since  1.0.0
 * @return RealHomes_Property_Feed
 */
function RHPF() {
    return RealHomes_Property_Feed::instance();
}

RHPF();